/* $Header: <fileName>.c	<lastVersionNumber>    	<lastEditionDate> 
 * =============================================================================
 *
 *            TEXAS INSTRUMENTS INCORPORATED PROPRIETARY INFORMATION
 *
 *  Property of Texas Instruments
 *  For Unrestricted Internal Use Only 
 *  Unauthorized reproduction and/or distribution is strictly
 *  prohibited.
 *  This product is protected under copyright law and trade secret law  
 *  as an unpublished work.
 *  Created 2003, (C) Copyright 2003 Texas Instruments.  All rights 
 *  reserved.
 */
/**
 *  @Component      PCT_PADTEST_TC
 *
 *  @Filename       PCT_PADTEST_TC.c
 *
 *  @Description    Primary test case source file for this component. See primary 
 *                  test case component header file for description of test case 
 *                  component.
 *
 *//*======================================================================== */

/* ============================================================================
* STANDARD INCLUDE FILES
* =============================================================================
*/

/* ============================================================================
* PROJECT SPECIFIC INCLUDE FILES
* =============================================================================
*/
#include "GlobalTypes.h"
#include "baseaddress.h"
#include "icpusync.h"
#include "cram.h"
#include "RESExt.h"
#ifdef __DMLED__
#include "res_dmled.h"
#endif

#include "trm_global_types.h"

/* 
   trm_register_access.h defines read32,read16, write16 and write32 macros
   which are used to access the device Control Module registers
*/
  
#include "trm_register_access.h"
#include "PCT_PADTEST_TC.h"
#include "DEBUGIO.h"


 
/* ============================================================================
* GLOBAL VARIABLES DECLARATIONS
* =============================================================================
*/

/* ============================================================================
* LOCAL TYPES AND DEFINITIONS
* =============================================================================
*/


/* ============================================================================
* LOCAL VARIABLES DECLARATIONS
* =============================================================================
*/

/* ============================================================================
* LOCAL FUNCTIONS PROTOTYPES
* =============================================================================
*/

/* ============================================================================
* FUNCTIONS
* =============================================================================
*/

/*
 ===========================================================================
 *************************************************************** 
   Function Non_Padconf_Reg_Update  is to update additional 
   (non-padconfiguration registers) associated to a pad.
   Here masks are applied to protect bits of other pads against 
   modification, during current pad setting.
   Note that masking is not done for 16-bit padconf register  
 ***************************************************************
============================================================================ 
*/
 
 /* 
  BaseAddr, temp_Reg_OffsetAddr,temp_Reg_Value,temp_Reg_Mask 
  are variables (with values actual for different registers)  
   taken from PCT header, and stored in structure array "temp"  variable
  Variable  n matches current index of additional register for the pad
 */




int  Non_Padconf_Reg_Update (u32 BaseAddr,u32 temp_Reg_OffsetAddr,u32 temp_Reg_Value,u32 temp_Reg_Mask,int n)

{ 
   u32 masked_val;      /// masked value is the value read from Register during program execution  AND-ed with inverted mask(mask which is generated by PCT)
   u32 PCT_masked_val;  /// PCT_masked_val is the actual value to OR with inverted mask(mask generated by PCT) just before writing to Register 


                      /// Criteria for non-existing pad Registers from Register_i to Register_10 
		      /// is temp_Reg_OffsetAddr=0x0 for i-th register (where i=1 to 10) which is considered
		      /// to mean that parameters for all registers from i-th to 10-th are 0    

     if (temp_Reg_OffsetAddr!=0)
 
         {
      
   
            PCT_masked_val = temp_Reg_Value & temp_Reg_Mask; /// The PCT_masked_val is equal to  reg value extracted from PCT (considering also PCT
                                                             /// generated  power-on reset values) ANDed with the Register Mask Vector generated from PCT
                                                             /// 
						             /// The PCT-generated 32-bit mask vector contains '1' ONLY at positions
						             /// which are modified by PCT pad controller	corresponding to adjusted pad					  
							      /// Unmodified bit positions in PCT-generated mask contain '0' 
						   
  
  
            masked_val = (read32(BaseAddr,temp_Reg_OffsetAddr)) & ~temp_Reg_Mask; /// masked_val is the value read 
                                                                                  /// from Register during customer program execution  
									          /// AND-ed with inverted (PCT-generated 32-bit mask vector)
  
  
            masked_val |= PCT_masked_val; /// masked_val is then OR-ed with PCT_masked_val to obtain
                                      /// final value to write in the pad setting register: Register_i 
  
  
            write32 (BaseAddr,temp_Reg_OffsetAddr,masked_val);
  
  
            fprintf (UARTW," The Non-padconf Register%d having offset %x  is set to %x in PCT,\r\n ",n ,temp_Reg_OffsetAddr, read32(BaseAddr,temp_Reg_OffsetAddr));
  
            return (1);
         }
    
              else
                      {  fprintf (UARTW," Register%d up to Register10 NOT defined for the pad \r\n ", n);
       
                         return (0);
                      }  


 }


/*
 ===========================================================================
   Function Pad_Regs_Write transfers PCT structure parameters to Padconf 
   and up to 10 additional  registers.
============================================================================ 
*/

 int  Pad_Regs_Write (PADCONF_padInfo_t *p)
 
 { 
 
  int reg_availbl; /// This flag receives status returned from Non_Padconf_Reg_Update function 
                   
  
  fprintf (UARTW," The PCT configured pad is  %s \r\n ",p->pad_name);
 
  /* 
    Updates value for up to 10  registers (different than padconf register) for additional pad functionality
    settings like I/O cell LOAD settings, MODE settings, Drive strength, Slew Rate Control, Weak Driver, Impedance,
    Dual voltage, Capacitance Load adjustments,Transmission line length, etc. 
    The size of list of additional registers associated  to a pad, varies from pad to pad 
    With this pad test program the supported maximal number of registers is 10 which provides that all registers modified 
    by PCT for pad are covered (for all pads).
    If a Register with certain number (1-10) does not exist, initialization to {0} performed in top test function 
    provides that all its parameters by default are 0. In this case its Offset address value is illegal =0x0  
    
  */     
                                 
				 /* The reg_availbl flag is 0 if Register_i (i=1 to 10) does not exist for currently adjusted pad */
                               

                               

				 
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg1_OffsetAddr, p->Reg1_Value, p->Reg1_Mask, 1);
                                                
                                 
				 /* chain-skip Non_Padconf_Reg_Update func execution  for not-available registers */
     
     if (reg_availbl)                           
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg2_OffsetAddr, p->Reg2_Value, p->Reg2_Mask, 2);
                                                
						
     if (reg_availbl)
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg3_OffsetAddr, p->Reg3_Value, p->Reg3_Mask, 3);
 
     if (reg_availbl)
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg4_OffsetAddr, p->Reg4_Value, p->Reg4_Mask, 4);
 
     if (reg_availbl)
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg5_OffsetAddr, p->Reg5_Value, p->Reg5_Mask, 5);
 
     if (reg_availbl)
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg6_OffsetAddr, p->Reg6_Value, p->Reg6_Mask, 6);
 
     if (reg_availbl)
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg7_OffsetAddr, p->Reg7_Value, p->Reg7_Mask, 7);
 
     if (reg_availbl)
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg8_OffsetAddr, p->Reg8_Value, p->Reg8_Mask, 8);
 
     if (reg_availbl)
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg9_OffsetAddr, p->Reg9_Value, p->Reg9_Mask, 9);
 
     if (reg_availbl)
     reg_availbl = Non_Padconf_Reg_Update (p->BaseAddr, p->Reg10_OffsetAddr, p->Reg10_Value, p->Reg10_Mask, 10);
 
 
     /* Some pads - like lpddr21,lpddr22 i/f pads don't have padconf registers in Control Module
      PCT returns -1 for the offset address of non-existant padconf register */
      
     if ((p->Padconf_OffsetAddr) == -1 ) 
     
     fprintf (UARTW," This pad does not have associated padconf register,\r\n ");
  
           else
                
		{ 
                     write16(p->BaseAddr, p->Padconf_OffsetAddr, p->Padconf_Value);
         
                     fprintf (UARTW," The padconf is defined and its value is set to %x in PCT,\r\n ",read16 (p->BaseAddr, p->Padconf_OffsetAddr));
                 }  
  
 
 
   
   }





/* 
   =============================================================
   ****************** Primary Test function*********************
          This function  contains structure initialization 
        to 0x0 and to list of values defined in  PCT header
   ============================================================= 
 */

ReturnCode_t PCT_PADTEST_TC_TestFunction( void )
{
    ReturnCode_t checkReturn = RET_FAIL;
#ifndef __DMLED__
    ReturnCode_t SlaveErrorCode;
#endif
    /* Test Specific Variable definition */
/*                                            */


/* add Test Specific  Variable definition here */
/*                                            */

// Initialize whole structure variable  to zero up to Register 10
   
   u16 i=0;
   PADCONF_padInfo_t temp[NUM_OF_PADS] = {{0}}, *PADCONF_padInfo;
   PADCONF_padInfo_t pct_init[NUM_OF_PADS] = PAD_ARRAY;  
   
   /* Pad registers assignment loop here */
   
   for (i=0;i<NUM_OF_PADS;i++)
      { 
         temp[i] = pct_init[i]; /* Array of structures fields  will be updated 
	                           Only in pad associated registers listed from PCT ,
                                   other register fields are kept 0 */  
         
      }	 
  
   
   
   /* Loop to configure registers for all listed pads */
   	 
      for (i=0;i<NUM_OF_PADS;i++)
      
      { 
         PADCONF_padInfo=&temp[i];
         Pad_Regs_Write (PADCONF_padInfo); 
      }	 
  
   
   
 

    /* Start Slave CPU's if Needed */
#ifndef __DMLED__
    ICPUSYNC_RunTC();
#endif

   

    /* Stop Slave CPU's if needed */
#ifndef __DMLED__
    SlaveErrorCode = ICPUSYNC_WaitTCStatus();
    if ( SlaveErrorCode != RET_OK )
    {
        RES_Set(SlaveErrorCode);
        checkReturn = RET_FAIL;
    }
#endif

    return checkReturn;

} /* PCT_PADTEST_TC_TestFunction() */




/*
void PCT_PADTEST_TC_IntHandler( UWORD16 inInterrupt )
{

    return;

} 
*/
/* PCT_PADTEST_TC_IntHandler() */



/* ============================================================================
* LOCAL FUNCTIONS
* =============================================================================
*/

/* EOF */
